"""
Wrapper for retrieving configurations and safely logging their retrieval
"""
import re

from pydantic import BaseModel
from pydantic import BaseSettings
from pydantic import Field

from dkist_service_configuration.logging import logger


class ConfigurationBase(BaseSettings):
    """Settings base which logs configured settings while censoring secrets"""

    log_level: str = Field("INFO", env="LOGURU_LEVEL")
    # env -> validation_alias in pydantic 2.x

    class Config:
        env_file = ".env"
        env_file_encoding = "utf-8"

    # model_config = SettingsConfigDict(env_file=".env", env_file_encoding="utf-8") in pydantic 2.x

    @staticmethod
    def _is_secret(field_name: str) -> bool:
        for pattern in ("pass", "secret", "token"):
            if re.search(pattern, field_name):
                return True
        return False

    def log_configurations(self):
        for field_name in self.__fields__:  # __fields__ -> model_fields in pydantic 2.x
            if self._is_secret(field_name=field_name):
                logger.info(f"{field_name}: <CENSORED>")
            else:
                logger.info(f"{field_name}: {getattr(self, field_name)}")


class MeshService(BaseModel):
    """Model of the metadata for a node in the service mesh"""

    host: str = Field(..., alias="mesh_address")
    port: int = Field(..., alias="mesh_port")


class MeshServiceConfigurationBase(ConfigurationBase):
    """
    Settings base for services using a mesh configuration to define connections in the form
    {
        "upstream_service_name": {"mesh_address": "localhost", "mesh_port": 6742}
    }
    """

    service_mesh: dict[str, MeshService] = Field(
        default_factory=dict, env="MESH_CONFIG"
    )  # env -> validation_alias when going to pydantic 2.x

    def service_mesh_detail(self, service_name) -> MeshService | None:
        return self.service_mesh.get(service_name)



"""Tests for the logging module"""
from dkist_service_configuration import logger


def test_log_levels():
    logger.trace("trace")
    logger.debug("debug")
    logger.info("info")
    logger.warning("warning")
    logger.error("error")



"""
Standard logging configuration for which intercepts standard logger messages from imported
libraries to emit through the loguru handler.  Must be the first import on the entry point to
ensure the first execution of the standard library logger basicConfig method
"""
import logging
from os import environ
from sys import stderr

from loguru import logger

__all__ = ["logger"]

LOG_LEVEL_TO_NAME = {
    5: "TRACE",
    10: "DEBUG",
    20: "INFO",
    25: "SUCCESS",
    30: "WARNING",
    40: "ERROR",
    50: "CRITICAL",
}
LOG_NAME_TO_LEVEL = {v: k for k, v in LOG_LEVEL_TO_NAME.items()}

# Retrieve default log level from same environment variable as loguru
log_level = LOG_NAME_TO_LEVEL.get(environ.get("LOGURU_LEVEL", "DEBUG"))

# Turn off better exceptions for log levels above debug to prevent secret leaking
is_better_exceptions_active = log_level <= 10

# Remove the default stderr handler
logger.remove()

# Add back the stderr handler with the diagnose flag set
logger.add(stderr, diagnose=is_better_exceptions_active)


class InterceptHandler(logging.Handler):
    """
    Handler to route stdlib logs to loguru
    """

    def emit(self, record):
        # Retrieve context where the logging call occurred, this happens to be in the 6th frame upward
        logger_opt = logger.opt(depth=6, exception=record.exc_info)
        # Log with name to support formatting if known, otherwise use the level number
        logger_opt.log(LOG_LEVEL_TO_NAME.get(record.levelno, record.levelno), record.getMessage())


# Configuration for stdlib logger to route messages to loguru; must be run before other imports
logging.basicConfig(handlers=[InterceptHandler()], level=log_level)
